import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import processing.sound.*; 
import fisica.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class Solid_Chirp_GGJ2017 extends PApplet {

//import processing.serial.*;

static final int STATE_INIT = -1;
static final int STATE_INTRO = 0;
static final int STATE_MENU = 1;
static final int STATE_RUNNING = 2;

static final int PHASE_SHOWING = 0;
static final int PHASE_RECORDING = 1;
static final int PHASE_LIVE = 2;

boolean[] keys = new boolean[2];
int state = STATE_INIT;
int phase = PHASE_SHOWING; 

int scalesCounter = 0;
float[] scales = new float[140];

PImage beccaccia, mollica, introBG, scoreBG;
PImage[] counter = new PImage[3];
static final int TIME = 30;
int currentTime = TIME;
PFont currentFont, biggerFont, mediumFont;
public void setup() {
  
  //size(1024, 768, P2D);
  //if (false)
  //initController();
  
  introBG = loadImage("intro.png");
  scoreBG = loadImage("score.png");
  beccaccia = loadImage("beccaccia.png");
  counter[0] = loadImage("1.png");
  counter[1] = loadImage("2.png");
  counter[2] = loadImage("3.png");
  mollica = loadImage("seme.png");
  currentFont = createFont("Verdana", 32);
  biggerFont = createFont("Verdana", 302);
  mediumFont = createFont("Verdana", 96);
  textFont(currentFont);
  imageMode(CENTER);
  textAlign(CENTER, CENTER);
  noStroke();
  thread("initGame");
}

public void initGame() {
  state = STATE_INIT;
  initAudio();
  initPhysics();
  currentTime = TIME;
  score = 0;
  state = STATE_INTRO;
  direction = 1;
}
int direction = 1;

public void displayPlayer() {
  pushMatrix();
  translate(c.getX(), c.getY() - 15);
  scale(direction, 1);
  image(beccaccia, 0, 0);
  fill(255, 255);
  if (currentTime > TIME - 4) {/*
    fill(255);
    ellipse(0, -80, 40, 40);
    triangle(-10, -80, 10, -80, 0, -30);
    fill(80);
    text(currentTime - (TIME - 4), 0, - 80);
    */
    image(counter[currentTime -TIME + 3], 40, -45);
    
  }
  popMatrix();
}

public void draw() {
  switch(state) {
  case STATE_INIT:
    background(0);
    break;
  case STATE_INTRO:
    background(66);
    image(introBG, width / 2, height / 2);
    break;
  case STATE_MENU:
     background(66);
    image(scoreBG, width / 2, height / 2);
    textFont(mediumFont);
    fill(128);
    text(score + "/10", width / 2 + 230, height / 2 - 30);
    break;
  case STATE_RUNNING:
    updatePlayer();
    switch (phase) {
    case PHASE_SHOWING:
      scalesCounter = 0;
      displayBackground();
      world.step();
      world.draw();
      displayBonus();
      displayPlayer();
      if (currentTime < TIME - 3)
        phase = PHASE_RECORDING;
      break;
    case PHASE_RECORDING:
      displayBackground();

      world.step();
      world.draw();
      float scale= rms.analyze();
      if (scalesCounter < scales.length) {
        scales[scalesCounter] = scale;
        float[] tempScales = new float[scalesCounter + 1];
        for (int i = 0; i < tempScales.length; i++)
          tempScales[i] = scales[i];

        drawPoly(tempScales);
        scalesCounter++;
      } else {
        addPoly(scales);
        phase = PHASE_LIVE;
      }
      displayBonus();
      displayPlayer();
      break;
    case PHASE_LIVE:      
      displayBackground();

      world.step();
      world.draw();
      displayBonus();
      displayPlayer();
      break;
    }
    fill(255);
    text(score, 70, 40);

    break;
  }
}

public void displayBackground() {
  background(125, 255, 125);
  if (currentTime > TIME - 4)
    return;
  pushStyle();
  pushMatrix();
  translate(width / 2, height * 0.2f);
  textFont(biggerFont);
  fill(255, 70);
  text(currentTime, 0, 0);
  textFont(currentFont);
  popMatrix();
  popStyle();
}

public void displayBonus() {
  for (int i = 0; i < bonus.length; i++)
    if (bonus[i] != null)
      image(mollica, bonus[i].x, bonus[i].y);
}

public void updatePlayer() {
  if (c.isTouchingBody(poly) || c.isTouchingBody(b)) {
    if (keys[1]) {
      direction = 1;
      c.addImpulse(345, -345);
      keys[1] = false;
    } else if (keys[0]) {
      direction = -1;
      c.addImpulse(-345, -345);
      keys[0] = false;
    }
  } else {
    if (keys[1]) {
      direction = 1;
      c.addImpulse(145, -50);
      keys[1] = false;
    } else if (keys[0]) {
      direction = -1;
      c.addImpulse(-145, -50);
      keys[0] = false;
    }
  }
  for (int i = 0; i < bonus.length; i++)
    if (bonus[i] != null && dist(bonus[i].x, bonus[i].y, c.getX(), c.getY()) < 60) {
      bonus[i] = null;
      score += 1;
    }
  if (millis() - lastTime > 1000) {
    currentTime--;
    lastTime = millis();
  }
  if (currentTime < 1)
    state = STATE_MENU;
}
long lastTime = 0;
int score = 0;
public void keyPressed() {
  switch(state) {
  case STATE_INIT:
    break;
  case STATE_INTRO:
    if (key == ' ') {
      state = STATE_RUNNING;
      phase = PHASE_SHOWING;
    }
    break;
  case STATE_MENU:
    if (key == ' ')
      thread("initGame");
    break;
  case STATE_RUNNING:
    switch (phase) {
    case PHASE_SHOWING:
      break;
    case PHASE_RECORDING:

      break;
    case PHASE_LIVE:
      if (keyCode == LEFT) {
        keys[0] = true;
      } else if (keyCode == RIGHT) {
        keys[1] = true;
      }
      if (key == 'r')
        thread("initGame");
      //world.remove(world.bottom);
      break;
    }
    break;
  }
}

public void keyReleased() {
  if (keyCode == LEFT) {
    keys[0] = false;
  } else if (keyCode == RIGHT) {
    keys[1] = false;
  }
}



AudioIn input;
Amplitude rms;

public void initAudio() {
  //Create an Audio input and grab the 1st channel
  input = new AudioIn(this, 0);

  // start the Audio Input
  input.start();

  // create a new Amplitude analyzer
  rms = new Amplitude(this);

  // Patch the input to an volume analyzer
  rms.input(input);
  input.amp(0.1f);
}
/*Serial myPort;
String inputString;

void initController() {
 myPort = new Serial(this, Serial.list()[0], 115200); 
}
void serialEvent(Serial p) { 
  inputString = p.readStringUntil('\n');
  
  if (inputString == null)
    return;
    
    inputString = inputString.substring(0, 2);
    //println(inputString.length());
  if (inputString.equals("sx")) {
    keys[0] = true;
    //println("Sinistra");
  }
  if (inputString.equals("dx"))
    keys[1] = true;
  //println(inputString);
} 
*/


FWorld world;
FCircle c;
PVector[]  bonus = new PVector[10];
FBox b;
public void initPhysics() {

  Fisica.init(this);
  world = new FWorld();
  world.setGravity(0, 800);
  world.setEdges();
  //world.remove(world.left);
  //world.remove(world.right);
  world.remove(world.top);
  b = new FBox(50, 150);
  c = new FCircle(25);
  c.setPosition(30, height - 300);
  c.setDensity(1);
  c.setRestitution(0.1f);
  c.setRotatable(false);
  c.setBullet(true);
  c.setNoStroke();
  //c.attachImage(beccaccia);
  world.add(c);
  b.setPosition(25, height - 75);
  b.setStatic(true);
  b.setNoStroke();
  world.add(b);
  for (int i = 0; i < bonus.length; i++)
    bonus[i] = new PVector(random(80, width), random(height * 0.3f, height * 0.7f));
    
    
  rectWidth = (float) (width - 50) / (scales.length - 1);
}
float rectWidth;
FPoly poly;

float altitude = 350;
float maxAltitude = 2000;
public void addPoly(float[] sample) {
  poly = new FPoly();
  poly.setFill(255, 128, 0);
  sample[0] = 0;
  poly.vertex(50, altitude - 50);
  //float rectWidth = (float) (width - 50) / (sample.length - 1);
  for (int i = 1; i < sample.length; i++) {
    poly.vertex(50 + (i * rectWidth), altitude - (sample[i] * maxAltitude));//, rectWidth, scales[i]);
  }
  for (int i = sample.length - 1; i > 0; i--) {
    poly.vertex(50 + (i * rectWidth), altitude + 100);//, rectWidth, scales[i]);
  }
  poly.vertex(50, altitude + 100);
  poly.setDensity(0.5f);
  poly.setRestitution(0.2f);
  poly.setNoStroke();
  world.add(poly);
}

public void drawPoly(float[] sample) {
  beginShape();
  fill(220);
  sample[0] = 0;
  vertex(50, altitude - 50);
  for (int i = 1; i < sample.length; i++) {
    vertex(50 + (i * rectWidth), altitude - (sample[i] * maxAltitude));//, rectWidth, scales[i]);
  }
  for (int i = sample.length - 1; i > 0; i--) {
    vertex(50 + (i * rectWidth), altitude + 100);//, rectWidth, scales[i]);
  }

  vertex(50, altitude + 100);
  endShape();
}
  public void settings() {  fullScreen(P2D); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "Solid_Chirp_GGJ2017" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
